"use strict";
var __createBinding = (this && this.__createBinding) || (Object.create ? (function(o, m, k, k2) {
    if (k2 === undefined) k2 = k;
    Object.defineProperty(o, k2, { enumerable: true, get: function() { return m[k]; } });
}) : (function(o, m, k, k2) {
    if (k2 === undefined) k2 = k;
    o[k2] = m[k];
}));
var __exportStar = (this && this.__exportStar) || function(m, exports) {
    for (var p in m) if (p !== "default" && !exports.hasOwnProperty(p)) __createBinding(exports, m, p);
};
var __awaiter = (this && this.__awaiter) || function (thisArg, _arguments, P, generator) {
    function adopt(value) { return value instanceof P ? value : new P(function (resolve) { resolve(value); }); }
    return new (P || (P = Promise))(function (resolve, reject) {
        function fulfilled(value) { try { step(generator.next(value)); } catch (e) { reject(e); } }
        function rejected(value) { try { step(generator["throw"](value)); } catch (e) { reject(e); } }
        function step(result) { result.done ? resolve(result.value) : adopt(result.value).then(fulfilled, rejected); }
        step((generator = generator.apply(thisArg, _arguments || [])).next());
    });
};
Object.defineProperty(exports, "__esModule", { value: true });
exports.BaseProcess = void 0;
const streams_1 = require("../../streams");
const git_1 = require("../../utils/git");
class BaseProcess {
    constructor() {
        this.lcovPath = [];
        this.opts = {
            stream: {},
        };
        this.lcov = { $: { linesCovered: 0, linesValid: 0 }, detail: {} };
        this.formatData = {
            total: { line: 0, covLine: 0, rate: '' },
            files: [],
        };
    }
    format() {
        this.formatData.total.covLine = this.lcov.$.linesCovered;
        this.formatData.total.line = this.lcov.$.linesValid;
        this.formatData.total.rate = `${((this.lcov.$.linesCovered / this.lcov.$.linesValid) *
            100).toFixed(2)}%`;
        Object.entries(this.lcov.detail).forEach(([name, detail]) => {
            const temp = {
                name,
                line: detail.linesValid,
                covLine: detail.linesCovered,
                rate: `${(detail.lineRate * 100).toFixed(2)}%`,
                detail: [],
            };
            temp.detail = detail.lines.map((item) => {
                return {
                    number: +item.number,
                    hits: item.hits,
                };
            });
            this.formatData.files.push(temp);
        });
    }
    output(data) {
        var _a, _b;
        let tempStream;
        if (((_a = this.opts.stream) === null || _a === void 0 ? void 0 : _a.name) === 'file') {
            tempStream = new streams_1.FileStream(data, this.opts.stream.opts);
        }
        else if (((_b = this.opts.stream) === null || _b === void 0 ? void 0 : _b.name) === 'stdio') {
            tempStream = new streams_1.StdoutStream(data, this.opts.stream.opts);
        }
        else {
            throw new Error('参数错误');
        }
        tempStream.outToStream();
    }
    getCreateInfo() {
        return __awaiter(this, void 0, void 0, function* () {
            return git_1.getGitRepoFirstCommitInfo(this.opts.cwd);
        });
    }
    getGitRepoInfo() {
        const remoteUrl = git_1.getGitRepoRemoteUrl(this.opts.cwd);
        const branch = git_1.getGitRepoCurrentBranch(this.opts.cwd);
        return {
            remoteUrl,
            branch
        };
    }
}
exports.BaseProcess = BaseProcess;
__exportStar(require("./full"), exports);
__exportStar(require("./increase"), exports);
